------------------------------------------------------------------------
-- CPE 269 VHDL File: clk_div.vhd
-- Experiment 2: used to divide the clock from the 50MHz provided on the 
--               board to something mere humans can perceive. 
--
-- Author: bryan mealy  (06-01-04)
--
-- revisions: 
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

-------------------------------------------------------------------------------
-- Clock divider circuit: 1) div by 2^12 (clk_fast_out) 
--                        2) div by 2^25 (clk_slow_out) 
-------------------------------------------------------------------------------
entity clk_div is
    Port (          CLK : in std_logic;
           clk_slow_out : out std_logic;
           clk_fast_out : out std_logic);
end clk_div;


---------------------------------------------------------------------------
architecture my_clk_div of clk_div is

   -- clock buffer declaration ------------------
   component BUFGP
      port (I : in std_logic; 
            O : out std_logic); 
   end component;

   -- this is here to get rid of a meaningless warning message 
   attribute box_type : string; 
   attribute box_type of BUFGP : component is "black_box"; 

   -- clock divider element declation -----------   
   component clk_div_element
      port ( CLK_in, RESET : in std_logic;
             CLK_slow_out  : out std_logic);
   end component;


   -- intermediate signal declaration -----------------------
   signal tclk1, tclk2, tclk3, tclk4, tclk5 : std_logic; 
   signal buf_clk1, buf_clk2, buf_sig : std_logic; 

begin

   -- div by 2^12 for fast clock -------------------
   clk_fast_out <= tclk3; 

   -- instantiate the clock buffer -----------------
   U1: BUFGP port map (I => CLK, O => buf_sig);

   -- instantiate the clock divider elements -------
   CLKDV1 : clk_div_element port map (CLK_IN=> buf_sig, RESET=> '0', CLK_slow_out => tclk1); 
   CLKDV2 : clk_div_element port map (CLK_IN=> tclk1,   RESET=> '0', CLK_slow_out => tclk2); 
   CLKDV3 : clk_div_element port map (CLK_IN=> tclk2,   RESET=> '0', CLK_slow_out => tclk3); 
   CLKDV4 : clk_div_element port map (CLK_IN=> tclk3,   RESET=> '0', CLK_slow_out => tclk4); 
   CLKDV5 : clk_div_element port map (CLK_IN=> tclk4,   RESET=> '0', CLK_slow_out => tclk5); 
   CLKDV6 : clk_div_element port map (CLK_IN=> tclk5,   RESET=> '0', CLK_slow_out => buf_clk1); 

   -- final div by 2 for slow clock output ---------
   div_by_2: process (buf_clk1)
   begin
       if (rising_edge(buf_clk1)) then 
          buf_clk2 <= not (buf_clk2); 
       end if; 
   end process div_by_2; 

   -- div by 2^25 for fast clock -------------------
   clk_slow_out <= buf_clk2; 

end my_clk_div;




------------------------------------------------------------------------
-- CPE 269 VHDL File: clk_div_elem.vhd
-- Description: a single clock divider element comprised of a 4-bit 
--               counter. Not too exciting.  
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-----------------------------------------------------------------------
-- interface of the 4-bit counter
-----------------------------------------------------------------------
entity clk_div_element is
    Port ( CLK_in, RESET : in std_logic;
           CLK_slow_out  : out std_logic);
end clk_div_element;

-----------------------------------------------------------------------
-- implementation of a 4-bit counter 
-----------------------------------------------------------------------
architecture my_clk_div_elem of clk_div_element is
begin
   process(CLK_in, RESET) 
      variable count : INTEGER range 0 to 255 := 0; 
   begin
      if (RESET = '1') then 
         count := 0; 
         CLK_slow_out <= '0'; 
      elsif (rising_edge(CLK_in)) then 
	      if (count = 15) then 
	         count := 0; 
		   else 
		      count := count + 1; 		    
		   end if; 
		 
         -- adjust for a 50% duty cycle ----------------
         if ( (count > 7) ) then CLK_slow_out <= '0'; 
		   else CLK_slow_out <= '1'; 
		   end if; 
      
      end if; 
   end process; 
end my_clk_div_elem;

